@@ -50,6 +50,11 @@
 #'   the corresponding posterior predictive distribution. If not given, we obtain
 #'   the posterior distribution of the parameters in \code{object}. See Prediction
 #'   section below.
+#' @param NE.include Logical value. If \code{TRUE}, non-estimable columns are
+#'   kept but returned as columns of \code{NA} values (this may create errors or
+#'   warnings in subsequent analyses using, say, \pkg{coda}). If \code{FALSE},
+#'   non-estimable columns are dropped, and a warning is issued. (If all are
+#'   non-estimable, an error is thrown.)
 #' @param ... arguments passed to other methods
 #'
 #' @return An object of class \code{\link[coda]{mcmc}} or \code{\link[coda]{mcmc.list}}.
@@ -103,11 +108,30 @@
 #' # (perhaps a bias adjustment should be applied; see "sophisticated" vignette)
 #' pred.incidence <- as.mcmc(regrid(cbpp.rg), likelihood = "binomial", trials = 20)
 as.mcmc.emmGrid = function(x, names = TRUE, sep.chains = TRUE, 
-                           likelihood, ...) {
+                           likelihood, NE.include = FALSE, ...) {
     if (is.na(x@post.beta[1])) {
         stop("No posterior sample -- can't make an 'mcmc' object")
     }
-    mat = x@post.beta %*% t(x@linfct)
+# notes on estimabilityn issues:
+# 1. Use @bhat to determine which coefs to use
+# 2. @nabasis as in freq models
+# 3. @post.beta we will EXCLUDE cols corresp to NAs in @bhat
+# See stanreg support for hints/details
+    use = which(!is.na(x@bhat))
+    est = estimability::is.estble(x@linfct, x@nbasis)
+    if (!any(est))
+        stop("Aborted -- No estimates in the grid are estimable")
+    else if(!all(est) && !NE.include) {
+        rows = paste(which(!est), collapse = ", ")
+        warning("Cases  ", rows, "  were dropped due to non-estimability", call. = FALSE)
+    }
+    mat = x@post.beta %*% t(x@linfct[, use, drop = FALSE])
+    if (NE.include)
+        mat[, !est] = NA
+    else {
+        mat = mat[, est, drop = FALSE]
+        x@grid = x@grid[est, , drop = FALSE]
+    }
     if(!is.null(offset <- x@grid[[".offset."]])) {
         n = nrow(mat)
         mat = mat + matrix(rep(offset, each = n), nrow = n)
@@ -169,7 +193,7 @@ as.mcmc.list.emmGrid = function(x, names = TRUE, ...) {
 #' 
 #' This function computes point estimates and HPD intervals for each
 #' factor combination in \code{object@emmGrid}. While this function
-#' may be called independently, it is called utomatically by the S3 method
+#' may be called independently, it is called automatically by the S3 method
 #' \code{\link{summary.emmGrid}} when the object is based on a Bayesian model.
 #' (Note: the \code{level} argument, or its default, is passed as \code{prob}).
 #'
@@ -253,8 +277,12 @@ hpd.summary = function(object, prob, by, type, point.est = median,
     
     
     ### OK, finally, here is the real stuff
-    mesg = misc$initMesg
-    mcmc = as.mcmc.emmGrid(object, names = FALSE, sep.chains = FALSE, ...)
+    pe.lbl = as.character(substitute(point.est))
+    if(length(pe.lbl) > 1) 
+        pe.lbl = "user-supplied function"
+    mesg = c(misc$initMesg, paste("Point estimate displayed:", pe.lbl))
+    mcmc = as.mcmc.emmGrid(object, names = FALSE, sep.chains = FALSE, 
+                           NE.include = TRUE, ...)
     mcmc = mcmc[, use.elts, drop = FALSE]
     if (inv) {
         if (bias.adjust) {
@@ -273,6 +301,8 @@ hpd.summary = function(object, prob, by, type, point.est = median,
     else if(!is.null(link))
         mesg = c(mesg, paste("Results are given on the", link$name, "(not the response) scale."))
     
+    est = !is.na(mcmc[1, ])
+    mcmc[, !est] = 0 # temp so we don't get errors
     mesg = c(mesg, paste("HPD interval probability:", prob))
     pt.est = data.frame(apply(mcmc, 2, point.est))
     names(pt.est) = object@misc$estName
@@ -294,6 +324,8 @@ hpd.summary = function(object, prob, by, type, point.est = median,
             names(pt.est) = misc$estName = "response"
     }
     
+    summ[!est, ] = NA
+    pt.est[!est, ] = NA
     summ = cbind(lbls, pt.est, summ)
     attr(summ, "estName") = misc$estName
     attr(summ, "clNames") = cnm
@@ -356,6 +388,18 @@ emm_basis.MCMCglmm = function(object, trms, xlev, grid, vcov.,
     if (mode == "multinomial") {
         misc$postGridHook = .MCMCglmm.multinom.postGrid
     }
+    else { # try to figure out the link
+        fam = unique(object$family)
+        if (length(fam) > 1)
+            stop("There is more than one 'family' in this model - too complex for emmeans support")
+        link = switch(fam,
+                      poisson = "log",
+                      multinomial = "log",
+                      categorical = "logit",
+                      ordinal = "logit") # maybe more later?
+        if (!is.null(link))
+            misc = .std.link.labels(list(link = link), misc)
+    }
     list(X = X, bhat = bhat, nbasis = matrix(NA), V = V, 
          dffun = function(k, dfargs) Inf, dfargs = list(), 
          misc = misc, post.beta = Sol)
@@ -363,7 +407,7 @@ emm_basis.MCMCglmm = function(object, trms, xlev, grid, vcov.,
 
 
 
-.MCMCglmm.multinom.postGrid = function(object) {
+.MCMCglmm.multinom.postGrid = function(object, ...) {
     linfct = object@linfct
     misc = object@misc
     post.lp = object@post.beta %*% t(linfct)
@@ -531,11 +575,21 @@ emm_basis.stanreg = function(object, trms, xlev, grid, mode, rescale, ...) {
             }
         }
         
-        misc$respName = as.character(terms(object))[2]
+        misc$respName = as.character.default(terms(object))[2]
     }
     samp = as.matrix(object$stanfit)[, names(bhat), drop = FALSE]
     attr(samp, "n.chains") = object$stanfit@sim$chains
-    list(X = X, bhat = bhat, nbasis = estimability::all.estble, V = V, 
+    # estimability...
+    nbasis = estimability::all.estble
+    all.nms = colnames(X)
+    if (length(names(bhat)) < (n <- length(all.nms))) {
+        coef = NA * X[1, ]
+        coef[names(bhat)] = bhat
+        bhat = coef
+        mmat = model.matrix(trms, object$data, contrasts.arg = contr)
+        nbasis = estimability::nonest.basis(mmat)
+    }
+    list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
          dffun = function(k, dfargs) Inf, dfargs = list(), 
          misc = misc, post.beta = samp)
 }
