@@ -1,5 +1,5 @@
 ##############################################################################
-#    Copyright (c) 2012-2017 Russell V. Lenth                                #
+#    Copyright (c) 2012-2019 Russell V. Lenth                                #
 #                                                                            #
 #    This file is part of the emmeans package for R (*emmeans*)              #
 #                                                                            #
@@ -35,21 +35,22 @@ contrast = function(object, ...)
 #' @rdname contrast
 #' @param object An object of class \code{emmGrid}
 #' @param method Character value giving the root name of a contrast method (e.g.
-#'   \code{"pairwise"} -- see \link{emmc-functions}). Alternatively, a named
-#'   \code{list} of coefficients (for a contrast or linear function) that must
-#'   each conform to the number of results in each \code{by} group. In a
-#'   multi-factor situation, the factor levels are combined and treated like a
-#'   single factor.
-#' @param interaction Character vector or logical value. If this is specified,
+#'   \code{"pairwise"} -- see \link{emmc-functions}). Alternatively, a function
+#'   of the same form, or a named \code{list} of coefficients (for a contrast or
+#'   linear function) that must each conform to the number of results in each
+#'   \code{by} group. In a multi-factor situation, the factor levels are
+#'   combined and treated like a single factor.
+#' @param interaction Character vector, logical value, or list. If this is specified,
 #'   \code{method} is ignored. See the \dQuote{Interaction contrasts} section
 #'   below for details.
 #' @param by Character names of variable(s) to be used for ``by'' groups. The
 #'   contrasts or joint tests will be evaluated separately for each combination
 #'   of these variables. If \code{object} was created with by groups, those are
 #'   used unless overridden. Use \code{by = NULL} to use no by groups at all.
-#' @param offset Numeric vector of the same length as each \code{by} group.
-#'   These values are added to their respective linear estimates. (It is ignored
-#'   when \code{interaction} is specified.)
+#' @param offset,scale Numeric vectors of the same length as each \code{by} group.
+#'   The \code{scale} values, if supplied, multiply their respective linear estimates, and
+#'   any \code{offset} values are added. Scalar values are also allowed. 
+#'   (These arguments are ignored when \code{interaction} is specified.)
 #' @param name Character name to use to override the default label for contrasts
 #'   used in table headings or subsequent contrasts of the returned object.
 #' @param options If non-\code{NULL}, a named \code{list} of arguments to pass
@@ -62,6 +63,17 @@ contrast = function(object, ...)
 #'   \code{by}, or a list thereof. See the section below on simple contrasts.
 #' @param combine Logical value that determines what is returned when
 #'   \code{simple} is a list. See the section on simple contrasts.
+#' @param ratios Logical value determining how log and logit transforms are
+#'   handled. These transformations are exceptional cases in that there is a
+#'   valid way to back-transform contrasts: differences of logs are logs of
+#'   ratios, and differences of logits are odds ratios. If \code{ratios = TRUE}
+#'   and summarized with \code{type = "response"}, \code{contrast} results are
+#'   back-transformed to ratios whenever we have true contrasts (coefficients
+#'   sum to zero). For other transformations, there is no natural way to
+#'   back-transform contrasts, so even when summarized with \code{type = "response"},
+#'   contrasts are computed and displayed on the linear-predictor scale. Similarly, 
+#'   if \code{ratios = FALSE}, log and logit transforms are treated in the same way as
+#'   any other transformation.
 #' @param ... Additional arguments passed to other methods
 #'
 #' @return \code{contrast} and \code{pairs} return an object of class
@@ -76,16 +88,21 @@ contrast = function(object, ...)
 #'   "revpairwise")}.
 #'
 #' @section Interaction contrasts: When \code{interaction} is specified,
-#'   interaction contrasts are computed: Contrasts are generated for each factor
-#'   separately, one at a time; and these contrasts are applied to the object
-#'   (the first time around) or to the previous result (subsequently). (Any
-#'   factors specified in \code{by} are skipped.) The final result comprises
-#'   contrasts of contrasts, or, equivalently, products of contrasts for the
-#'   factors involved. Processing is done in the order of appearance in
-#'   \code{object@levels}. With \code{interaction = TRUE}, \code{method} (if
-#'   specified as character) is used for each contrast. If \code{interaction} is
-#'   a character vector, the elements specify the respective contrast method(s);
-#'   they are recycled as needed.
+#'   interaction contrasts are computed. Specifically contrasts are generated
+#'   for each factor separately, one at a time; and these contrasts are applied
+#'   to the object (the first time around) or to the previous result
+#'   (subsequently). (Any factors specified in \code{by} are skipped.) The final
+#'   result comprises contrasts of contrasts, or, equivalently, products of
+#'   contrasts for the factors involved. Any named elements of \code{interaction}
+#'   are assigned to contrast methods; others are assigned in order of
+#'   appearance in \code{object@levels}. The contrast factors in the resulting 
+#'   \code{emmGrid} object are ordered the same as in \code{interaction}.
+#'   
+#'   \code{interaction} may be a character vector or list of valid contrast
+#'   methods (as documented for the \code{method} argument). If the vector or
+#'   list is shorter than the number needed, it is recycled. Alternatively, if
+#'   the user specifies \code{contrast = TRUE}, the contrast specified in
+#'   \code{method} is used for all factors involved.
 #'   
 #' @section Simple contrasts:
 #'   \code{simple} is essentially the complement of \code{by}: When
@@ -138,17 +155,26 @@ contrast = function(object, ...)
 #' }
 #'
 #' # An interaction contrast for tension:wool
-#' tw.emm <- contrast(warp.emm, interaction = c("poly", "consec"), by = NULL)
+#' tw.emm <- contrast(warp.emm, interaction = c(tension = "poly", wool = "consec"), 
+#'                    by = NULL)
 #' tw.emm          # see the estimates
 #' coef(tw.emm)    # see the contrast coefficients
+#' 
+#' # Use of scale and offset
+#' #   an unusual use of the famous stack-loss data...
+#' mod <- lm(Water.Temp ~ poly(stack.loss, degree = 2), data = stackloss)
+#' (emm <- emmeans(mod, "stack.loss", at = list(stack.loss = 10 * (1:4))))
+#' # Convert results from Celsius to Fahrenheit:
+#' confint(contrast(emm, "identity", scale = 9/5, offset = 32))
+#' 
 contrast.emmGrid = function(object, method = "eff", interaction = FALSE, 
-                        by, offset = NULL, name = "contrast", 
+                        by, offset = NULL, scale = NULL, name = "contrast", 
                         options = get_emm_option("contrast"), 
-                        type, adjust, simple, combine = FALSE, ...) 
+                        type, adjust, simple, combine = FALSE, ratios = TRUE, ...) 
 {
     if(!missing(simple))
         return(.simcon(object, method = method, interaction = interaction,
-                      offset = offset, name = name, options = options,
+                      offset = offset, scale = scale, name = name, options = options,
                       type = type, simple = simple, combine = combine, 
                       adjust = adjust, ...))
     if(missing(by)) 
@@ -166,29 +192,47 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     if (is.logical(interaction) && interaction)
         interaction = method
     if (!is.logical(interaction)) { # i.e., interaction is not FALSE
-        if (!is.character(interaction))
-            stop("interaction requires named contrast function(s)")
         if(missing(adjust))
             adjust = "none"
-        ### by = NULL why was this here before ???
         vars = names(object@levels)
         k = length(vars)
         if(!is.null(by)) {
             vars = c(setdiff(vars, by), by)
             k = k - length(by)
         }
-        interaction = rep(interaction, k)[1:k]
+        nms = names(interaction)
+        interaction = as.list(rep(interaction, k)[1:k])
+        names(interaction) = c(nms, rep("", k))[1:k]
+        nms = vars[1:k]
+        if (is.null(names(interaction)))
+            names(interaction) = nms
+        else {
+            unnamed = which(!(names(interaction) %in% nms))
+            names(interaction)[unnamed] = setdiff(nms, names(interaction))
+            nms = names(interaction)
+        }
+        
+        
+        # if (!is.character(interaction))
+        #     stop("interaction requires named contrast function(s)")
+        ### by = NULL why was this here before ???
         tcm = NULL
         for (i in k:1) {
-            nm = paste(vars[i], interaction[i], sep = "_")
-            object = contrast.emmGrid(object, interaction[i], by = vars[-i], name = nm, ...)
+            if (is.character(interaction[[i]]))
+                nm = paste(nms[i], interaction[[i]], sep = "_")
+            else
+                nm = paste(nms[i], "custom", sep = "_")
+            pos = which(vars == nms[i])
+            object = contrast.emmGrid(object, interaction[[i]], by = vars[-pos], name = nm, ...)
             if(is.null(tcm))
                 tcm = object@misc$con.coef
             else
                 tcm = object@misc$con.coef %*% tcm
-            vars[i] = nm
+            vars[pos] = nm
         }
-        object = update(object, by = by, adjust = adjust, ..., silent = TRUE)
+        object = update(object, by = by, adjust = adjust, silent = TRUE) 
+          ### removed `...` here Nov 2019 because a `mode` arg gets matched with `model.info`
+          ### when passed via formula lhs in `emmeans()`
         object@misc$is.new.rg = NULL
         object@misc$orig.grid = orig.grid
         object@misc$con.coef = tcm
@@ -216,8 +260,9 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
             args[[nm]] = NULL
             all.args[[nm]] = NULL
         }
-        all.levs = do.call("paste", all.args)   # keep all levels in case we have permutations of them
+        all.levs = do.call("paste", unname(all.args))   # keep all levels in case we have permutations of them
     }
+    args = unname(args)
     args$sep = ","
     levs = do.call("paste", args)  # NOTE - these are levels for the first (or only) by-group
     if (length(levs) == 0)   # prevent error when there are no levels to contrast
@@ -228,7 +273,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
         cmat = as.data.frame(method, optional = TRUE)
         # I have no clue why they named that argument 'optional',
         # but setting it to TRUE keeps it from messing up the names
-        method = function(levs) cmat
+        method = function(levs, ...) cmat
     }
     else if (is.character(method)) {
         fn = paste(method, "emmc", sep=".")
@@ -239,7 +284,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     }
     # case like in old lsmeans, contr = list
     else if (!is.function(method))
-        stop("'method' must be a function or the basename of an '.emmc' function")
+        stop("'method' must be a list, function, or the basename of an '.emmc' function")
     
     # Get the contrasts; this should be a data.frame
     cmat = method(levs, ...)
@@ -252,6 +297,13 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     else if (nrow(cmat) != nrow(args))
         stop("Nonconforming number of contrast coefficients")
     tcmat = t(cmat)
+    if (!is.null(scale)) {
+        if (length(scale) %in% c(1, nrow(tcmat)))
+            tcmat = tcmat * scale
+        else
+            stop("'scale' length of ", length(scale), 
+                 " does not conform with ", nrow(tcmat), " contrasts", call. = FALSE)
+    }
     
     if (is.null(by)) {
         linfct = tcmat %*% linfct
@@ -275,7 +327,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
                 tmp$con[seq_along(cm) + length(cm)*(i-1)] = names(cm)
             }
         }
-        grid = data.frame(.contrast. = tmp$con)
+        grid = data.frame(.contrast. = factor(tmp$con, levels = unique(tmp$con)))
         n.each = ncol(cmat)
         row.1st = sapply(by.rows, function(x) x[1])
         xlevs = list()
@@ -297,7 +349,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
     
     row.names(linfct) = NULL
     misc = object@misc
-    misc$initMesg = NULL # initial annotation likely will no longer apply
+    misc$initMesg = NULL
     misc$estName = "estimate"
     if (!is.null(et <- attr(cmat, "type")))
         misc$estType = et
@@ -306,7 +358,7 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
         misc$estType = ifelse(is.con, "contrast", "prediction")
     }
     misc$methDesc = attr(cmat, "desc")
-    misc$famSize = size = length(by.rows[[1]])
+    misc$famSize = ifelse(is.null(fs <- attr(cmat, "famSize")), length(by.rows[[1]]), fs)
     misc$pri.vars = setdiff(names(grid), c(".offset.",".wgt."))
     if (missing(adjust)) adjust = attr(cmat, "adjust")
     if (is.null(adjust)) adjust = "none"
@@ -317,6 +369,8 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
             grid[[".offset."]] = 0
         grid[[".offset."]] = grid[[".offset."]] + rep(offset, length(by.rows))
     }
+    if (!is.null(fs <- attr(cmat, "famSize")))
+        misc$famSize = fs
     misc$is.new.rg = FALSE
     misc$adjust = adjust
     misc$infer = c(FALSE, TRUE)
@@ -327,26 +381,36 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
         by.cols[unlist(by.rows)] = by.cols # gives us inverse of by.rows order
     misc$orig.grid = orig.grid  # save original grid
     misc$con.coef = tcmat[ , by.cols, drop = FALSE] # save contrast coefs
+    # test that each set of coefs sums to 0
+    true.con = all(abs(apply(cmat, 2, function(.) sum(.) / (1.0e-6 + max(abs(.))))) < 1.0e-6) 
+    if(is.na(true.con)) # prevent error when there are no contrasts
+        true.con = FALSE
+    if(true.con)
+        misc$sigma = NULL   # sigma surely no longer makes sense
+
     # zap the transformation info except in special cases
     if (!is.null(misc$tran)) {
-        misc$orig.tran = misc$tran
-        true.con = all(zapsmall(apply(cmat, 2, sum)) == 0) # each set of coefs sums to 0
-        if(is.na(true.con)) # prevent error when there are no contrasts
-            true.con = FALSE
-        if (true.con && misc$tran %in% c("log", "genlog", "logit")) {
+        misc$orig.tran = .fmt.tran(misc)
+        if (ratios && true.con && misc$tran %in% c("log", "genlog", "logit")) {
             misc$log.contrast = TRUE      # remember how we got here; used by summary
             misc$orig.inv.lbl = misc$inv.lbl
             if (misc$tran == "logit") {
                 misc$inv.lbl = "odds.ratio"
                 misc$tran = "log.o.r."
+                misc$tran.mult = misc$tran.offset = NULL
             }
             else {
                 misc$inv.lbl = "ratio"
                 misc$tran = "log"
+                misc$tran.mult = misc$tran.offset = NULL
             }
         }
-        else
-            misc$tran = misc$tran.mult = NULL
+        else {
+            misc$initMesg = c(misc$initMesg, 
+                              paste("Note: contrasts are still on the", misc$orig.tran, "scale"))
+            message("Note: Use 'contrast(regrid(object), ...)' to obtain contrasts of back-transformed estimates")
+            misc$tran = misc$tran.mult = misc$tran.offset = NULL
+        }
     }
     
     # ensure we don't inherit inappropriate settings
@@ -419,7 +483,6 @@ contrast.emmGrid = function(object, method = "eff", interaction = FALSE,
 #' @rdname contrast 
 #' @param x An \code{emmGrid} object
 #' @param reverse Logical value - determines whether to use \code{"pairwise"} (if \code{TRUE}) or \code{"revpairwise"} (if \code{FALSE}).
-#' @inheritParams contrast.emmGrid 
 #' @importFrom graphics pairs
 #' @export
 pairs.emmGrid = function(x, reverse = FALSE, ...) {
