@@ -49,13 +49,8 @@ str.emmGrid <- function(object, ...) {
         if (is.null(x)) cat("(predicted by other variables)")
         else cat(paste(format(x, digits = 5, justify = "none"), collapse=", "))
     }
-    showtran = function(tran, label) { # internal convenience fcn
-        if (is.list(tran)) 
-            tran = ifelse(is.null(tran$name), "custom", tran$name)
-        if (!is.null(mul <- object@misc$tran.mult))
-            tran = paste0(mul, "*", tran)
-        cat(paste(label, dQuote(tran), "\n"))
-        
+    showtran = function(misc, label) { # internal convenience fcn
+        cat(paste(label, dQuote(.fmt.tran(misc)), "\n"))
     }
     levs = object@levels
     cat(paste("'", class(object)[1], "' object with variables:\n", sep=""))
@@ -87,9 +82,9 @@ str.emmGrid <- function(object, ...) {
         cat("\n")
     }
     if(!is.null(tran <- object@misc$tran)) {
-        showtran(tran, "Transformation:")
+        showtran(object@misc, "Transformation:")
         if (!is.null(tran2 <- object@misc$tran2))
-            showtran(tran2, "Additional response transformation:")
+            showtran(list(tran = tran2), "Additional response transformation:")
     }
 }
 
@@ -109,7 +104,7 @@ print.emmGrid = function(x,...)
 #' @param object An \code{emmGrid} object
 #' @param ... (required but not used)
 #' 
-#' @return The \code{vcov} method returns a ymmetric matrix of variances and
+#' @return The \code{vcov} method returns a symmetric matrix of variances and
 #'   covariances for \code{predict.emmGrid(object, type = "lp")}
 #'
 #' @method vcov emmGrid
@@ -176,6 +171,10 @@ vcov.emmGrid = function(object, ...) {
 #' for that matter), we should have \code{tran = "sqrt"} and \code{tran.mult =
 #' 2}. If absent, a multiple of 1 is assumed.}
 #' 
+#' \item{\code{tran.offset}}{Additive constant before a transformation is applied.
+#' For example, a response transformation of \code{log(y + pi)} has
+#' \code{tran.offset  = pi}. If no value is present, an offset of 0 is assumed.}
+#' 
 #' \item{\code{estName}}{(\code{character}) is the column label used for
 #' displaying predictions or EMMs.}
 #' 
@@ -191,8 +190,10 @@ vcov.emmGrid = function(object, ...) {
 #' levels are used as columns in each table produced by \code{\link{summary.emmGrid}}.}
 #' 
 #' \item{\code{alpha}}{(numeric) is the default significance level for tests, in
-#' \code{\link{summary.emmGrid}} as well as \code{\link{CLD.emmGrid}} and \code{\link{plot.emmGrid}}
-#' when \samp{intervals = TRUE}}
+#' \code{\link{summary.emmGrid}} as well as \code{\link{plot.emmGrid}}
+#' when \samp{CIs = TRUE}. Be cautious that methods that depend on
+#' specifying \code{alpha} are prone to abuse. See the
+#' discussion in \href{../doc/basics.html#pvalues}{\code{vignette("basics", "emmeans")}}.}
 #' 
 #' \item{\code{adjust}}{(\code{character)}) is the default for the \code{adjust}
 #' argument in \code{\link{summary.emmGrid}}.}
@@ -202,14 +203,15 @@ vcov.emmGrid = function(object, ...) {
 #' along with \code{"adjust"} to determine appropriate adjustments to P values
 #' and confidence intervals.}
 #' 
-#' \item{\code{famSize}}{(integer) is the \code{nmeans} parameter for
-#' \code{\link{ptukey}} when \code{adjust="tukey"}. }
+#' \item{\code{famSize}}{(integer) is the number of means involved in a family of
+#' inferences; used in Tukey adjustment}
 #' 
 #' \item{\code{infer}}{(\code{logical} vector of length 2) is the default value
 #' of \code{infer} in \code{\link{summary.emmGrid}}.}
 #' 
 #' \item{\code{level}}{(numeric) is the default confidence level, \code{level},
-#' in \code{\link{summary.emmGrid}}}
+#' in \code{\link{summary.emmGrid}}. \emph{Note:} You must specify all five letters 
+#' of \sQuote{level} to distinguish it from the slot name \sQuote{levels}.}
 #' 
 #' \item{\code{df}}{(numeric) overrides the default degrees of freedom with a
 #' specified single value.}
@@ -220,6 +222,9 @@ vcov.emmGrid = function(object, ...) {
 #' \item{\code{side}}{(numeric or character) \code{side} specification for for
 #' \code{summary} or \code{test} (taken to be zero if missing).}
 #' 
+#' \item{\code{sigma}}{(numeric) Error SD to use in predictions and for bias-adjusted
+#' back-transformations}
+#' 
 #' \item{\code{delta}}{(numeric) \code{delta} specification for \code{summary}
 #' or \code{test} (taken to be zero if missing).}
 #' 
@@ -247,13 +252,22 @@ vcov.emmGrid = function(object, ...) {
 #' documentation for \code{\link{ref_grid}}. The current nesting structure is
 #' displayed by \code{\link{str.emmGrid}}.}
 #' 
-#' \item{(any slot name)}{If the name matches an element of
-#' \code{slotNames(object)}, that slot is replaced by the supplied value, if it
-#' is of the required class (otherwise an error occurs). Note that other options
-#' above are saved in the \code{misc} slot; hence, you probably
-#' don't want to replace that slot. The user must be very careful in
-#' replacing slots because they are interrelated; for example, the \code{levels}
-#' and \code{grid} slots must involve the same variable names, and the lengths
+#' \item{\code{levels}}{named \code{list} of new levels for the elements of the
+#' current \code{emmGrid}. The list name(s) are used as new variable names, and
+#' if needed, the list is expanded using \code{expand.grid}. These results replace
+#' current variable names and levels. This specification changes the \code{levels}, 
+#' \code{grid}, \code{roles}, and \code{misc} slots in the updated \code{emmGrid},
+#' and resets \code{pri.vars}, \code{by.vars}, \code{adjust}, \code{famSize},
+#' \code{avgd.over}, and \code{nesting}. 
+#' \emph{Note:} All six letters of \code{levels} is needed in order to distinguish
+#' it from \code{level}.}
+#' 
+#' \item{(any other slot name)}{If the name matches an element of
+#' \code{slotNames(object)} other than \code{levels}, that slot is replaced by 
+#' the supplied value, if it is of the required class (otherwise an error occurs). 
+#' 
+#' The user must be very careful in
+#' replacing slots because they are interrelated; for example, the lengths
 #' and dimensions of \code{grid}, \code{linfct}, \code{bhat}, and \code{V} must
 #' conform.}
 #' } %%%%%%% end \describe 
@@ -273,7 +287,7 @@ update.emmGrid = function(object, ..., silent = FALSE) {
     valid.misc = c("adjust","alpha","avgd.over","by.vars","delta","df",
                    "initMesg","estName","estType","famSize","infer","inv.lbl",
                    "level","methDesc","nesting","null","predict.type","pri.vars"
-                   ,"side","tran","tran.mult","tran2","type","is.new.rg")
+                   ,"side","sigma","tran","tran.mult","tran.offset","tran2","type","is.new.rg")
     valid.slots = slotNames(object)
     valid.choices = union(valid.misc, valid.slots)
     misc = object@misc
@@ -286,7 +300,26 @@ update.emmGrid = function(object, ..., silent = FALSE) {
         }
         else {
             if (fullname == "type") fullname = "predict.type"
-            if (fullname %in% valid.slots)
+            if (fullname == "levels") {
+                lvls = args[[nm]]
+                if (!is.list(lvls))
+                    stop("'levels' must be a named list.")
+                nm = names(lvls)
+                if (is.null(nm) || any(nm == ""))
+                     stop("'levels' must be a named list.")
+                grd = do.call(expand.grid, lvls)
+                if (nrow(object@grid) != nrow(grd))
+                    stop("Length of replacement levels does not match the number of rows in the grid")
+                object@levels = lvls
+                for (nm in c(".wgt.", ".offset"))
+                        grd[[nm]] = object@grid[[nm]]
+                object@grid = grd
+                object@roles$predictors = misc$pri.vars = names(lvls)
+                misc$by.vars = misc$avgd.over = object@model.info$nesting = NULL
+                misc$adjust = "none"
+                misc$famSize = nrow(grd)
+            }
+            else if (fullname %in% valid.slots) # all slots but "levels"
                 slot(object, fullname) = args[[nm]]
             else {
                 if (fullname == "by.vars") {
@@ -297,7 +330,7 @@ update.emmGrid = function(object, ..., silent = FALSE) {
                     allvars = union(misc$pri.vars, misc$by.vars)
                     misc$by.vars = setdiff(allvars, args[[nm]])
                 }
-                # special case - I keep nesting in model.info. Plus add'kl checks
+                # special case - I keep nesting in model.info. Plus add'l checks
                 if (fullname == "nesting") {
                     object@model.info$nesting = lst = .parse_nest(args[[nm]])
                     if(!is.null(lst)) {
@@ -337,6 +370,8 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   \code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{test.emmGrid}},
 #'   \code{\link{confint.emmGrid}}, and \code{\link{emmip}}. The only option that can
 #'   affect the latter four is \code{"predict.method"}.}
+#' \item{\code{cov.keep}}{The default value of \code{cov.keep} in \code{\link{ref_grid}}.
+#'   Defaults to \code{"2"}, i.e., two-level covariates are treated like factors.}
 #' \item{\code{graphics.engine}}{A character value matching 
 #'   \code{c("ggplot", "lattice")}, setting the default engine to use in
 #'   \code{\link{emmip}} and \code{\link{plot.emmGrid}}.  Defaults to \code{"ggplot"}.}
@@ -360,6 +395,12 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   displayed is just enough to reasonably distinguish estimates from the ends
 #'   of their confidence intervals; but always at least 3 digits. If
 #'   \code{FALSE}, the system value \code{getOption("digits")} is used.}
+#' \item{\code{back.bias.adj}}{A logical value controlling whether we 
+#'   try to adjust bias when back-transforming. If \code{FALSE}, we use naive
+#'   back transformation. If \code{TRUE} \emph{and \code{sigma} is available}, a
+#'   second-order adjustment is applied to estimate the mean on the response
+#'   scale.}
+#'   
 #' }%%% end describe{}
 #' Some other options have more specific purposes:
 #' \describe{
@@ -440,12 +481,14 @@ emm_defaults = list (
     emmeans = list(infer = c(TRUE, FALSE)),
     contrast = list(infer = c(FALSE, TRUE)),
     save.ref_grid = TRUE,     # save new ref_grid in .Last.ref_grid
+    cov.keep = "2",           # default for cov.keep arg in ref_grid
     graphics.engine = "ggplot",  # default for emmip and plot.emmGrid
 ###    msg.data.call = TRUE,     # message when there's a call in data or subset
     msg.interaction = TRUE,   # message about averaging w/ interactions
     msg.nesting = TRUE,       # message when nesting is detected
     estble.tol = 1e-8,        # tolerance for estimability checks
     simplify.names = TRUE,    # simplify names like data$x to just "x"
+    back.bias.adj = FALSE,    # Try to bias-adjust back-transformations?
     opt.digits = TRUE,        # optimize displayed digits?
     lmer.df = "kenward-roger",  # Use Kenward-Roger for df
     disable.pbkrtest = FALSE, # whether to bypass pbkrtest routines for lmerMod
@@ -460,7 +503,7 @@ emm_defaults = list (
 ### Primary reason to do this is with transform = TRUE, then can 
 ### work with linear functions of the transformed predictions
 
-#' Reconstruct a reference grid with a new transformation
+#' Reconstruct a reference grid with a new transformation or posterior sample
 #' 
 #' The typical use of this function is to cause EMMs to be computed on
 #' a different scale, e.g., the back-transformed scale rather than the 
@@ -478,32 +521,67 @@ emm_defaults = list (
 #' contrasts will be conducted on the new scale -- which is
 #' the reason this function exists. 
 #' 
-#' In cases where the
-#' degrees of freedom depended on the linear function being estimated, the d.f.
-#' from the reference grid are saved, and a kind of \dQuote{containment} method
-#' is substituted in the returned object whereby the calculated d.f. for a new
-#' linear function will be the minimum d.f. among those having nonzero
-#' coefficients. This is kind of an \emph{ad hoc} method, and it can
-#' over-estimate the degrees of freedom in some cases.
-#'
+#' This function may also be used to convert a reference grid for a 
+#' frequentist model to one for a Bayesian model. To do so, specify a value
+#' for \code{N.sim} and a posterior sample is simulated using the function \code{sim}.
+#' . The grid may be further processed in accordance with
+#' the other arguments; or if \code{transform = "pass"}, it is simply returned with the 
+#' only change being the addition of the posterior sample.
+#' 
 #' @param object An object of class \code{emmGrid}
-#' @param transform Character or logical value. If \code{"response"} or
-#'   \code{"mu"}, the inverse transformation is applied to the estimates in the
-#'   grid (but if there is both a link function and a response transformation,
-#'   \code{"mu"} back-transforms only the link part); if \code{"log"}, the
-#'   results are formulated as if the response had been \code{log}-transformed;
-#'   if \code{"none"}, predictions thereof are on the same scale as in 
-#'   \code{object}, and any internal transformation information is preserved. 
-#'   For compatibility with past versions, \code{transform} may also be logical;
-#'   \code{TRUE} is taken as \code{"response"}, and \code{FALSE} as 
-#'   \code{"none"}.
-#' @param inv.log.lbl Character value. This applies only when \code{transform =
-#'   "log"}, and is used to label the predictions if subsequently summarized
+#' @param transform Character, list, or logical value. If \code{"response"},
+#'   \code{"mu"}, or \code{TRUE}, the inverse transformation is applied to the
+#'   estimates in the grid (but if there is both a link function and a response
+#'   transformation, \code{"mu"} back-transforms only the link part); if
+#'   \code{"none"} or \code{FALSE}, \code{object} is re-gridded so that its
+#'   \code{bhat} slot contains \code{predict(object)} and its \code{linfct} slot
+#'   is the identity. Any internal transformation information is preserved. If
+#'   \code{transform = "pass"}, the object is not re-gridded in any way (this
+#'   may be useful in conjunction with \code{N.sim}).
+#'   
+#'   If \code{transform} is a character value in \code{links} (which is the set
+#'   of valid arguments for the \code{\link{make.link}} function, excepting
+#'   \code{"identity"}), or if \code{transform} is a list of the same form as
+#'   returned by \code{make.links} or \code{\link{make.tran}}, the results are
+#'   formulated as if the response had been transformed with that link function.
+#'   
+#' @param inv.link.lbl Character value. This applies only when \code{transform} 
+#'   is in \code{links}, and is used to label the predictions if subsequently summarized
 #'   with \code{type = "response"}.
 #' @param predict.type Character value. If provided, the returned object is
-#'   updated with the given type, e.g., \code{"response"}. 
-#'   See \code{\link{update.emmGrid}}.
-#'   
+#'   updated with the given type to use by default by \code{summary.emmGrid}
+#'   (see \code{\link{update.emmGrid}}).  This may be useful if, for example,
+#'   when one specifies \code{transform = "log"} but desires summaries to be
+#'   produced by default on the response scale.
+#' @param bias.adjust Logical value for whether to adjust for bias in
+#'   back-transforming (\code{transform = "response"}). This requires a value of 
+#'   \code{sigma} to exist in the object or be specified.
+#' @param sigma Error SD assumed for bias correction (when 
+#'   \code{transform = "response"} and a transformation
+#'   is in effect). If not specified,
+#'   \code{object@misc$sigma} is used, and an error is thrown if it is not found.
+#' @param N.sim Integer value. If specified and \code{object} is based on a 
+#'   frequentist model (i.e., does not have a posterior sample), then a fake 
+#'   posterior sample is generated using the function \code{sim}.
+#' @param sim A function of three arguments (no names are assumed).
+#'   If \code{N.sim} is supplied with a frequentist model, this function is called
+#'   with respective arguments \code{N.sim}, \code{object@bhat}, and \code{object@V}.
+#'   The default is the multivariate normal distribution.
+#' @param ... Ignored.
+#' 
+#' @section Degrees of freedom:  
+#' In cases where the
+#' degrees of freedom depended on the linear function being estimated (e.g.,
+#' Satterthwaite method), the d.f.
+#' from the reference grid are saved, and a kind of \dQuote{containment} method
+#' is substituted in the returned object, whereby the calculated d.f. for a new
+#' linear function will be the minimum d.f. among those having nonzero
+#' coefficients. This is kind of an \emph{ad hoc} method, and it can
+#' over-estimate the degrees of freedom in some cases. An annotation is
+#' displayed below any subsequent summary results stating that the 
+#' degrees-of-freedom method is inherited from the previous method at
+#' the time of re-gridding.
+#'
 #' @note Another way to use \code{regrid} is to supply a \code{transform} 
 #'   argument to \code{\link{ref_grid}} (either directly of indirectly via
 #'   \code{\link{emmeans}}). This is often a simpler approach if the reference
@@ -514,25 +592,46 @@ emm_defaults = list (
 #'
 #' @examples
 #' pigs.lm <- lm(log(conc) ~ source + factor(percent), data = pigs)
+#' rg <- ref_grid(pigs.lm)
 #' 
 #' # This will yield EMMs as GEOMETRIC means of concentrations:
-#' emmeans(pigs.lm, "source", type = "response")
-#' # NOTE: pairs() of the above will be RATIOS of these results
+#' (emm1 <- emmeans(rg, "source", type = "response"))
+#' pairs(emm1) ## We obtain RATIOS
 #' 
 #' # This will yield EMMs as ARITHMETIC means of concentrations:
-#' emmeans(regrid(ref_grid(pigs.lm, transform = "response")), "source")
-#' # Same thing, made simpler:
-#' emmeans(pigs.lm, "source", transform = "response")
-#' # NOTE: pairs() of the above will be DIFFERENCES of these results
-regrid = function(object, transform = c("response", "mu", "unlink", "log", "none"), 
-                  inv.log.lbl = "response", predict.type) 
+#' (emm2 <- emmeans(regrid(rg, transform = "response"), "source"))
+#' pairs(emm2)  ## We obtain DIFFERENCES
+#' # Same result, useful if we hadn't already created 'rg'
+#' # emm2 <- emmeans(pigs.lm, "source", transform = "response")
+#' 
+#' # Simulate a posterior sample
+#' set.seed(2.71828)
+#' rgb <- regrid(rg, N.sim = 200, transform = "pass")
+#' emmeans(rgb, "source", type = "response")  ## similar to emm1
+regrid = function(object, transform = c("response", "mu", "unlink", "none", "pass", links), 
+                  inv.link.lbl = "response", predict.type, 
+                  bias.adjust = get_emm_option("back.bias.adj"), sigma, 
+                  N.sim, sim = mvtnorm::rmvnorm, ...) 
 {
+    links = c("logit", "probit", "cauchit", "cloglog", "log", "sqrt", "1/mu^2", "inverse")
     if (is.logical(transform))   # for backward-compatibility
         transform = ifelse(transform, "response", "none")
+    else if (is.list(transform)) {
+        userlink = transform
+        transform = "user"
+    }
     else
         transform = match.arg(transform)
     
-    # if we have two transformations to undo, do the first one recursively
+    if (is.na(object@post.beta[1]) && !missing(N.sim)) {
+        message("Generating a posterior sample of size ", N.sim)
+        object@post.beta = sim(N.sim, object@bhat, object@V)
+    }
+
+    if (transform == "pass")
+        return(object)
+    
+        # if we have two transformations to undo, do the first one recursively
     if ((transform == "response") && (!is.null(object@misc$tran2)))
         object = regrid(object, transform = "mu")
     
@@ -540,8 +639,11 @@ regrid = function(object, transform = c("response", "mu", "unlink", "log", "none
     PB = object@post.beta
     NC = attr(PB, "n.chains")
     
-    if (!is.na(PB[1])) # fix up post.beta BEFORE we overwrite parameters
+    if (!is.na(PB[1])) { # fix up post.beta BEFORE we overwrite parameters
         PB = PB %*% t(object@linfct)
+        if (".offset." %in% names(object@grid))
+            PB = t(apply(PB, 1, function(.) . + object@grid[[".offset."]]))
+    }
     
     est = .est.se.df(object, do.se = TRUE) ###FALSE)
     estble = !(is.na(est[[1]]))
@@ -558,6 +660,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "log", "none
     edf = df[estble]
     if (length(edf) == 0) edf = NA
     # note both NA/NA and Inf/Inf test is.na() = TRUE
+    prev.df.msg = attr(object@dffun, "mesg")
     if (any(is.na(edf/edf)) || (diff(range(edf)) < .01)) { # use common value
         object@dfargs = list(df = mean(edf, na.rm = TRUE))
         object@dffun = function(k, dfargs) dfargs$df
@@ -569,11 +672,25 @@ regrid = function(object, transform = c("response", "mu", "unlink", "log", "none
             ifelse(length(idx) == 0, NA, min(dfargs$df[idx], na.rm = TRUE))
         }
     }
+    if(!is.null(prev.df.msg)) 
+        attr(object@dffun, "mesg") = ifelse(
+            startsWith(prev.df.msg, "inherited"), prev.df.msg,
+                paste("inherited from", prev.df.msg, "when re-gridding"))
+
     
-    if(transform %in% c("response", "mu", "unlink", "log") && !is.null(object@misc$tran)) {
-        link = attr(est, "link")
-        D = .diag(link$mu.eta(object@bhat[estble]))
-        object@bhat = link$linkinv(object@bhat)
+    if(transform %in% c("response", "mu", "unlink", links, "user") && !is.null(object@misc$tran)) {
+        flink = link = attr(est, "link")
+        if (bias.adjust) {
+            if(missing(sigma))
+                sigma = object@misc$sigma
+            link = .make.bias.adj.link(link, sigma)
+            if (!is.na(PB[1])) # special frequentist version when sigma is MCMC sample
+                flink = .make.bias.adj.link(flink, mean(sigma))
+            else
+                flink = link
+        }
+        D = .diag(flink$mu.eta(object@bhat[estble]))
+        object@bhat = flink$linkinv(object@bhat)
         object@V = D %*% tcrossprod(object@V, D)
         if (!is.na(PB[1]))
             PB = matrix(link$linkinv(PB), ncol = ncol(PB))
@@ -586,32 +703,41 @@ regrid = function(object, transform = c("response", "mu", "unlink", "log", "none
         }
         if((transform %in% c("mu", "unlink")) && !is.null(object@misc$tran2)) {
             object@misc$tran = object@misc$tran2
-            object@misc$tran2 = object@misc$tran.mult = object@misc$inv.lbl = NULL
+            object@misc$tran2 = object@misc$tran.mult = object@misc$tran.offset = object@misc$inv.lbl = NULL
         }
         else
-            object@misc$tran = object@misc$tran.mult = object@misc$inv.lbl = NULL
+            object@misc$tran = object@misc$tran.mult = object@misc$tran.offset = object@misc$inv.lbl = NULL
+        sigma = object@misc$sigma = NULL
     }
-    if (transform == "log") { # from prev block, we now have stuff on response scale
-        Vee = vcov(object)
-        incl = which(object@bhat > 0)
+    if (transform %in% c(links, "user")) { # fake a transformation
+        link = if (transform == "user") userlink
+               else                     make.link(transform)
+        bounds = range(link$linkinv(c(-1e6, -100, -1, 0, 1, 100, 1e6)))
         nas = which(is.na(object@bhat)) # already NA
-        negs = which(object@bhat <= 0)
+        incl = vincl = which((object@bhat > bounds[1]) & (object@bhat < bounds[2]))
+        if (length(nas) > 0) 
+            vincl = which((object@bhat[-nas] > bounds[1]) & (object@bhat[-nas] < bounds[2]))
+        negs = setdiff(seq_along(object@bhat), incl)
         if (length(negs) > 0) {
-            message("Non-positive response predictions are flagged as non-estimable")
+            message("Invalid response predictions are flagged as non-estimable")
             object@bhat[negs] = NA
             tmp = seq_along(object@bhat)
             object@nbasis = sapply(c(nas, negs), function(ii) 0 + (tmp == ii))
         }
-        D = .diag(1/object@bhat[incl])
-        object@V = D %*% tcrossprod(Vee[incl, incl, drop = FALSE], D)
-        object@bhat = log(object@bhat)
+        object@bhat = link$linkfun(object@bhat)
+        Vee = object@V
+        if(length(incl) > 0) {
+            D = .diag(1/link$mu.eta(object@bhat[incl]))
+            object@V = D %*% tcrossprod(Vee[vincl, vincl, drop = FALSE], D)
+        }
         if (!is.na(PB[1])) {
             PB[PB <= 0] = NA
-            PB = log(PB)
+            PB = link$linkfun(PB)
             PB[1] = ifelse(is.na(PB[1]), 0, PB[1]) # make sure 1st elt isn't NA
         }
-        object@misc$tran = "log"
-        object@misc$inv.lbl = inv.log.lbl
+        object@misc$tran = if (transform == "user") link   
+                           else                     transform
+        object@misc$inv.lbl = inv.link.lbl
     }
     
     if(!is.na(PB[1])) {
