@@ -238,6 +238,7 @@ emmeans = function(object, specs, by = NULL,
     if(!is(object, "emmGrid")) {
         object = ref_grid(object, ...)
     }
+    options$type = list(...)$type  # note type if in ...
     if (is.list(specs)) {
         return (emmeans.list(object, specs, by = by, 
                              contr = contr, weights = weights, ...))
@@ -245,8 +246,8 @@ emmeans = function(object, specs, by = NULL,
     if (inherits(specs, "formula")) {
         spc = .parse.by.formula(specs)
         specs = spc$rhs
-        by = if (length(spc$by) == 0) NULL
-             else spc$by
+        if (length(spc$by) > 0) 
+            by = setdiff(union(spc$by, by), spc$rhs)
         if (length(spc$lhs) > 0) 
             contr = spc$lhs
     }
@@ -271,7 +272,7 @@ emmeans = function(object, specs, by = NULL,
         }
         
         # Ensure object is in standard order
-        ord = do.call(order, RG@grid[rev(names(RG@levels))])
+        ord = .std.order(RG@grid, RG@levels) ###do.call(order, unname(RG@grid[rev(names(RG@levels))]))
         if(any(ord != seq_along(ord)))
             RG = RG[ord]
         
@@ -570,6 +571,9 @@ emmobj = function(bhat, V, levels, linfct, df = NA, dffun, dfargs = list(),
 #'   argument is ignored in \code{as.list.emmGrid}
 #'   
 #' @return \code{as.emmGrid} returns an object of class \code{emmGrid}. 
+#'     However, in fact, both \code{as.emmGrid} and \code{as.emm_list} check for an
+#'     attribute in \code{object} to decide whether to return an \code{emmGrid} 
+#'     or \code{emm_list)} object.
 #' 
 #' @seealso \code{\link{emmobj}}
 #' @export
@@ -598,8 +602,12 @@ as.emmGrid = function(object, ...) {
             object$misc$is.new.rg = (cls == "ref.grid")
     }
     # above keeps us from having to define these classes in emmeans
-    if (is.list(object))
-        result = do.call(emmobj, object)
+    if (is.list(object)) {
+        if (!is.null(attr(object, "emm_list")))
+            return(as.emm_list(object))
+        else
+            result = do.call(emmobj, object)
+    }
     else {
         result = try(as(object, "emmGrid", strict = FALSE), silent = TRUE)
         if (inherits(result, "try-error"))
@@ -609,6 +617,7 @@ as.emmGrid = function(object, ...) {
 }
 
 #' @rdname as.emmGrid
+#' @order 2
 #' @param x An \code{emmGrid} object
 #' @return \code{as.list.emmGrid} returns an object of class \code{list}. 
 #' @method as.list emmGrid
@@ -639,5 +648,15 @@ as.list.emmGrid = function(x, ...) {
     return(FALSE)
 }
 
-
+### Sort grid in standard order according to ordering of entries in levels. 
+### Thus .std.order(do.call(expand.grid, levels), levels) --> 1,2,...,nrow
+.std.order = function(grid, levels) {
+    tmp = lapply(rev(names(levels)), function(nm) {
+        x = grid[[nm]]
+        if (inherits(x, "factor")) as.integer(x)
+        else as.integer(factor(x, levels = as.character(levels[[nm]])))
+        # Note: need as.character(levels) here so we handle such as Date vectors correctly
+    })
+    do.call(order, tmp)
+}
 
