@@ -103,8 +103,12 @@ recover_data.merMod = function(object, ...) {
 
 #' @export
 emm_basis.merMod = function(object, trms, xlev, grid, vcov., 
-                            mode = get_emm_option("lmer.df"), 
-                            lmer.df, ...) {
+                            mode = get_emm_option("lmer.df"), lmer.df, 
+                            disable.pbkrtest = get_emm_option("disable.pbkrtest"), 
+                            pbkrtest.limit = get_emm_option("pbkrtest.limit"), 
+                            disable.lmerTest = get_emm_option("disable.lmerTest"), 
+                            lmerTest.limit = get_emm_option("lmerTest.limit"), 
+                            options, ...) {
     if (missing(vcov.))
         V = as.matrix(vcov(object, correlation = FALSE))
     else
@@ -117,35 +121,43 @@ emm_basis.merMod = function(object, trms, xlev, grid, vcov.,
             mode = lmer.df
 
         mode = match.arg(tolower(mode), c("satterthwaite", "kenward-roger", "asymptotic"))
+        # if we're gonna override the df anyway, keep it simple 
+        # OTOH, if K-R, documentation promises we'll adjust V
+        if (!is.null(options$df) && (mode != "kenward-roger")) 
+            mode = "asymptotic"
+        
         
         # set flags
         objN = lme4::getME(object, "N")
-        disable.pbkrtest = get_emm_option("disable.pbkrtest")
-        tooBig.k = (objN > get_emm_option("pbkrtest.limit"))
-        disable.lmerTest = get_emm_option("disable.lmerTest")
-        tooBig.s = (objN > get_emm_option("lmerTest.limit"))
+        tooBig.k = (objN > pbkrtest.limit)
+        tooBig.s = (objN > lmerTest.limit)
         
         tooBigMsg = function(pkg, limit) {  
             message("Note: D.f. calculations have been",
                     " disabled because the number of observations exceeds ", limit, ".\n",
-                    "To enable adjustments, set emm_options(", pkg, ".limit = ", objN, ") or larger,\n",
+                    "To enable adjustments, add the argument '", pkg, ".limit = ", objN, "' (or larger)\n",
+                    "[or, globally, 'set emm_options(", pkg, ".limit = ", objN, ")' or larger];\n",
                     "but be warned that this may result in large computation time and memory use.")
         }
-        
+
         # pick the lowest-hanging apples first
         if (mode == "kenward-roger") {
-            if (disable.pbkrtest || tooBig.k || !requireNamespace("pbkrtest", quietly = TRUE))
+            if (disable.pbkrtest || tooBig.k || !.requireNS("pbkrtest", 
+                    "Cannot use mode = \"kenward-roger\" because *pbkrtest* package is not installed", 
+                    fail = message))
                 mode = "satterthwaite"
             if (!disable.pbkrtest && tooBig.k)
-                tooBigMsg("pbkrtest", get_emm_option("pbkrtest.limit"))
+                tooBigMsg("pbkrtest", pbkrtest.limit)
         }
         if (mode == "satterthwaite") {
-            if (disable.lmerTest || tooBig.s || !requireNamespace("lmerTest", quietly = TRUE))
+            if (disable.lmerTest || tooBig.s || !.requireNS("lmerTest", 
+                    "Cannot use mode = \"satterthwaite\" because *lmerTest* package is not installed", 
+                    fail = message))
                 mode = ifelse(!disable.pbkrtest && !tooBig.k && 
-                                  requireNamespace("pbkrtest", quietly = TRUE), 
+                                  .requireNS("pbkrtest", fail = .nothing), 
                               "kenward-roger", "asymptotic")
             if (!disable.lmerTest && tooBig.s)
-                tooBigMsg("lmerTest", get_emm_option("lmerTest.limit"))
+                tooBigMsg("lmerTest", lmerTest.limit)
         }
         # if my logic isn't flawed, we are guaranteed that mode is both desired and possible
         
@@ -179,7 +191,7 @@ emm_basis.merMod = function(object, trms, xlev, grid, vcov.,
             dffun = function(k, dfargs) Inf
         }
         
-        misc$initMesg = paste("Degrees-of-freedom method:", mode)
+        attr(dffun, "mesg") = mode
     }
     else if (lme4::isGLMM(object)) {
         dffun = function(k, dfargs) Inf
@@ -234,8 +246,15 @@ recover_data.lme = function(object, data, ...) {
 
 #' @export
 emm_basis.lme = function(object, trms, xlev, grid, 
-        mode = c("containment", "satterthwaite"), sigmaAdjust = TRUE, ...) {
+        mode = c("containment", "satterthwaite", "boot-satterthwaite", "auto"), 
+        sigmaAdjust = TRUE, options, ...) {
     mode = match.arg(mode)
+    if (!is.null(options$df)) # if we're gonna override the df anyway, keep it simple!
+        mode = "fixed"
+    if (mode == "auto")
+        mode = ifelse(is.null(object$apVar), "containment", "boot-satterthwaite")
+    if (is.null(object$apVar))
+        mode = "containment"
     contrasts = object$contrasts
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts)
@@ -249,8 +268,13 @@ emm_basis.lme = function(object, trms, xlev, grid,
     }
     nbasis = estimability::all.estble
     
-    if (mode == "satterthwaite") {
+    if (mode == "fixed") { # hack to just put in df from options
+        dfargs = list(df = options$df)
+        dffun = function(k, dfargs) dfargs$df
+    }
+    else if (mode %in% c("satterthwaite", "boot-satterthwaite")) {
         G = try(gradV.kludge(object), silent = TRUE)
+        ###! not yet, doesn't work G = try(lme_grad(object, object$call, object$data, V))
         if (inherits(G, "try-error"))
             stop("Unable to estimate Satterthwaite parameters")
         dfargs = list(V = V, A = object$apVar, G = G)
@@ -272,7 +296,7 @@ emm_basis.lme = function(object, trms, xlev, grid,
         }
         dfargs = list(dfx = dfx)
     }
-    misc$initMesg = paste("Degrees-of-freedom method:", mode)
+    attr(dffun, "mesg") = mode
     list(X = X, bhat = bhat, nbasis = nbasis, V = V, 
          dffun = dffun, dfargs = dfargs, misc = misc)
 }
@@ -282,19 +306,19 @@ emm_basis.lme = function(object, trms, xlev, grid,
 # model with a few random perturbations of y, then
 # regressing the changes in V against the changes in the 
 # covariance parameters
-gradV.kludge = function(object) {
+gradV.kludge = function(object, Vname = "varFix", call = object$call$fixed, data = object$data) {
     A = object$apVar
     theta = attr(A, "Pars")
-    V = object$varFix
+    V = object[[Vname]]
     sig = .01 * object$sigma
-    data = object$data
-    yname = all.vars(object$call$fixed)[1]
+    #data = object$data
+    yname = all.vars(call)[1]
     y = data[[yname]]
     n = length(y)
     dat = t(replicate(2 + length(theta), {
         data[[yname]] = y + sig * rnorm(n)
         mod = update(object, data = data)
-        c(attr(mod$apVar, "Pars") - theta, as.numeric(mod$varFix - V))
+        c(attr(mod$apVar, "Pars") - theta, as.numeric(mod[[Vname]] - V))
     }))
     dimnames(dat) = c(NULL, NULL)
     xcols = seq_along(theta)
@@ -303,8 +327,53 @@ gradV.kludge = function(object) {
     grad
 }
 
+# ### new way to get gradients for lme models
+# # (not ready for primetime...)
+# lme_grad = function(object, call, data, V) {
+#     obj = object$modelStruct
+#     conLin = object
+#     class(conLin) = class(obj)
+#     X = model.matrix(eval(call$fixed), data = data)
+#     y = data[[all.vars(call)[1]]]
+#     conLin$Xy = cbind(X, y)
+#     conLin$fixedSigma = FALSE
+#     grps = object$groups # May have to re-order these fancily?
+#     MEest = get("MEestimate", getNamespace("nlme")) ## workaround its not being exported
+#     func = function(x) {
+#         coef(obj) = x
+#         tmp = MEest(obj, grps, conLin)
+#         crossprod(tmp$sigma * tmp$varFix)
+#     }
+#     res = numDeriv::jacobian(func, coef(obj))
+#     G = lapply(seq_len(ncol(res)), function(j) matrix(res[, j], ncol = ncol(V)))
+#     G[[1 + length(G)]] = 2 * V  # gradient wrt log sigma
+#     G
+# }
+
+
 
 #--------------------------------------------------------------
+
+### new way to get gradients for gls models
+gls_grad = function(object, call, data, V) {
+    obj = object$modelStruct
+    conLin = object
+    class(conLin) = class(obj)
+    X = model.matrix(eval(call$model), data = data)
+    y = data[[all.vars(call)[1]]]
+    conLin$Xy = cbind(X, y)
+    conLin$fixedSigma = FALSE
+    func = function(x) {
+        obj = nlme::`coef<-`(obj, value = x)
+        tmp = nlme::glsEstimate(obj, conLin)
+        crossprod(tmp$sigma * tmp$varBeta)
+    }
+    res = numDeriv::jacobian(func, coef(obj))
+    G = lapply(seq_len(ncol(res)), function(j) matrix(res[, j], ncol = ncol(V)))
+    G[[1 + length(G)]] = 2 * V  # gradient wrt log sigma
+    G
+}
+
 ### gls objects (nlme package)
 recover_data.gls = function(object, ...) {
     fcall = object$call
@@ -314,16 +383,55 @@ recover_data.gls = function(object, ...) {
     recover_data(fcall, trms, object$na.action, ...)
 }
 
-emm_basis.gls = function(object, trms, xlev, grid, ...) {
+emm_basis.gls = function(object, trms, xlev, grid, 
+                         mode = c("auto", "df.error", "satterthwaite", "boot-satterthwaite"), 
+                         options, ...) {
     contrasts = object$contrasts
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = contrasts)
     bhat = coef(object)
     V = .my.vcov(object, ...)
     nbasis = estimability::all.estble
-    dfargs = list(df = object$dims$N - object$dims$p)
-    dffun = function(k, dfargs) dfargs$df
-    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=list())
+    misc = list()
+    mode = match.arg(mode)
+    if (!is.null(options$df)) # if we're gonna override the df anyway, keep it simple!
+        mode = "df.error"
+    if (mode == "auto")
+        mode = ifelse(is.null(object$apVar), "df.error", "satterthwaite")
+    if (!is.matrix(object$apVar))
+        mode = "df.error"
+    if (mode %in% c("satterthwaite", "boot-satterthwaite")) {
+        chk = attr(object$apVar, "Pars")
+        if(max(abs(coef(object$modelStruct) - chk[-length(chk)])) > .001) {
+            message("Analytical Satterthwaite method not available; using boot-satterthwaite")
+            mode = "boot-satterthwaite"
+        }
+        if (mode == "boot-satterthwaite") {
+            G = try(gradV.kludge(object, "varBeta", call = object$call,
+                                 data = eval(object$call$data)),
+                    silent = TRUE)
+        }
+        else
+            G = try(gls_grad(object, object$call, eval(object$call$data), V))
+        if (inherits(G, "try-error")) {
+            sugg = ifelse(mode == "satterthwaite", "boot-satterthwaite", "df.error")
+            stop("Can't estimate Satterthwaite parameters.\n",
+                 "  Try adding the argument 'mode = \"", sugg, "\"'", call. = FALSE)
+        }
+        dfargs = list(V = V, A = object$apVar, G = G)
+        dffun = function(k, dfargs) {
+            est = tcrossprod(crossprod(k, dfargs$V), k)
+            g = sapply(dfargs$G, function(M) tcrossprod(crossprod(k, M), k))
+            varest = tcrossprod(crossprod(g, dfargs$A), g)
+            2 * est^2 / varest
+        }
+    }
+    else if (mode == "df.error") {  ### mode == "df.error"
+        dfargs = list(df = object$dims$N - object$dims$p - length(attr(object$apVar, "Pars")))
+        dffun = function(k, dfargs) dfargs$df
+    }
+    attr(dffun, "mesg") = mode
+    list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
 }
 
 
@@ -366,7 +474,7 @@ emm_basis.polr = function(object, trms, xlev, grid,
             misc$postGridHook = ".clm.postGrid"
         }
     }
-    misc$respName = as.character(terms(object))[2]
+    misc$respName = as.character.default(terms(object))[2]
     nbasis = estimability::all.estble
     dffun = function(...) Inf
     list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=list(), misc=misc)
@@ -432,7 +540,7 @@ emm_basis.coxph = function (object, trms, xlev, grid, ...)
     result
 }
 
-.notran2 = function(object) {
+.notran2 = function(object, ...) {
     for (nm in c("tran", "tran2"))
         if(!is.null(object@misc[[nm]]) && object@misc[[nm]] == "Surv") object@misc[[nm]] = NULL
     object
