@@ -27,7 +27,8 @@
 #' Summaries, predictions, intervals, and tests for \code{emmGrid} objects
 #' 
 #' These are the primary methods for obtaining numerical or tabular results 
-#' from an \code{emmGrid} object.
+#' from an \code{emmGrid} object. Note that by default, summaries for Bayesian models are
+#' diverted to \code{\link{hpd.summary}}.
 #' 
 #' \code{summary.emmGrid} is the general function for summarizing \code{emmGrid} objects. 
 #' \code{confint.emmGrid} is equivalent to \code{summary.emmGrid with 
@@ -89,8 +90,17 @@
 #' @param frequentist Ignored except if a Bayesian model was fitted. If missing
 #'   or \code{FALSE}, the object is passed to \code{\link{hpd.summary}}. Otherwise, 
 #'   a logical value of \code{TRUE} will have it return a frequentist summary.
-#' @param ... (Not used by \code{summary.emmGrid} or \code{predict.emmGrid}.) In
-#'   \code{as.data.frame.emmGrid}, \code{confint.emmGrid}, and 
+#' @param bias.adjust Logical value for whether to adjust for bias in
+#'   back-transforming (\code{type = "response"}). This requires a value of 
+#'   \code{sigma} to exist in the object or be specified.
+#' @param sigma Error SD assumed for bias correction (when 
+#'   \code{type = "response"} and a transformation
+#'   is in effect), or for constructing prediction intervals. If not specified,
+#'   \code{object@misc$sigma} is used, and an error is thrown if it is not found.
+#'   \emph{Note:} \code{sigma} may be a vector, as long as it conforms to the number of rows
+#'   of the reference grid.
+#' @param ... (Not used by \code{summary.emmGrid}.) In
+#'   \code{as.data.frame.emmGrid}, \code{confint.emmGrid}, \code{predict.emmGrid}, and 
 #'   \code{test.emmGrid}, these arguments are passed to
 #'   \code{summary.emmGrid}.
 #'
@@ -119,6 +129,13 @@
 #'   on the scale of the linear predictor, not the inverse-transformed one.
 #'   Similarly, confidence intervals are computed on the linear-predictor scale,
 #'   then inverse-transformed.
+#'   
+#'   When \code{bias.adjust} is \code{TRUE}, then back-transformed estimates
+#'   are adjusted by adding 
+#'   \eqn{0.5 h''(u)\sigma^2}, where \eqn{h} is the inverse transformation and
+#'   \eqn{u} is the linear predictor. This is based on a second-order Taylor
+#'   expansion. There are better or exact adjustments for certain specific
+#'   cases, and these may be incorporated in future updates.
 #' 
 #' @section P-value adjustments:
 #'   The \code{adjust} argument specifies a multiplicity adjustment for tests or
@@ -212,6 +229,15 @@
 #'   \eqn{||Nx||^2 / ||x||^2} exceeds \code{tol}, which by default is
 #'   \code{1e-8}. You may change it via \code{\link{emm_options}} by setting
 #'   \code{estble.tol} to the desired value.
+#'   
+#' @section Warning about potential misuse of P values:
+#'   A growing consensus in the statistical and scientific community is that
+#'   the term \dQuote{statistical significance} should be completely abandoned, and
+#'   that criteria such as \dQuote{p < 0.05} never be used to assess the
+#'   importance of an effect. These practices are just too misleading and prone to abuse.
+#'   See \href{../doc/basics.html#pvalues}{the \dQuote{basics} vignette} for more
+#'   discussion.
+#'   
 #' 
 #' @note In doing testing and a transformation and/or link is in force, any
 #'   \code{null} and/or \code{delta} values specified must always be on the
@@ -227,6 +253,8 @@
 #'   unnecessary to call \code{summary} unless there is a need to
 #'   specify other than its default options.
 #'   
+#' @seealso \code{link{hpd.summary}}
+#' 
 #' @method summary emmGrid  
 #' @export
 #'
@@ -251,10 +279,15 @@
 #' test(contrast(pigs.emm, "consec"), joint = TRUE)
 #'
 summary.emmGrid <- function(object, infer, level, adjust, by, type, df, 
-                        null, delta, side, frequentist, ...) {
+                        null, delta, side, frequentist, 
+                        bias.adjust = get_emm_option("back.bias.adj"),
+                        sigma, ...) {
+    if(missing(sigma))
+        sigma = object@misc$sigma
 
     if(!is.na(object@post.beta[1]) && (missing(frequentist) || !frequentist))
-        return (hpd.summary(object, prob = level, by = by, type = type, ...))
+        return (hpd.summary(object, prob = level, by = by, type = type, 
+                            bias.adjust = bias.adjust, sigma = sigma, ...))
     
     # Any "summary" options override built-in
     opt = get_emm_option("summary")
@@ -263,6 +296,9 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
         object = do.call("update.emmGrid", opt)
     }
     
+    if(!missing(sigma))
+        object = update(object, sigma = sigma)  # we'll keep sigma in misc
+    
     misc = object@misc
     use.elts = .reconcile.elts(object)
     grid = object@grid[use.elts, , drop = FALSE]
@@ -293,8 +329,10 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
     
     if(missing(df)) 
         df = misc$df
-    if(!is.null(df))
-        object@dffun = function(k, dfargs) df
+    if(!is.null(df)) {
+        object@dffun = function(k, dfargs) df[1]
+        attr(object@dffun, "mesg") = "user-specified"
+    }
     
     # for missing args that default to zero unless provided or in misc slot
     .nul.eq.zero = function(val) {
@@ -316,7 +354,7 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
     side = side.map[pmatch(side, side.opts, 2)[1]] - 2
     delta = abs(delta)
     
-    result = .est.se.df(object)
+    result = .est.se.df(object, ...)
     
     lblnms = setdiff(names(grid), 
                      c(object@roles$responses, ".offset.", ".wgt."))
@@ -351,6 +389,10 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
     
     mesg = misc$initMesg
     
+    # Look for "mesg" attribute in dffun
+    if (!is.null(dfm <- attr(object@dffun, "mesg")))
+        mesg = c(mesg, paste("Degrees-of-freedom method:", dfm))
+    
     ### Add an annotation when we show results on lp scale and
     ### there is a transformation
     if (!inv && !is.null(link)) {
@@ -361,6 +403,8 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
         inv = FALSE
         link = NULL
     }
+    if(inv && bias.adjust && !is.null(link)) 
+        link = .make.bias.adj.link(link, sigma)
     
     # et = 1 if a prediction, 2 if a contrast (or unmatched or NULL), 3 if pairs
     et = pmatch(c(misc$estType, "c"), c("prediction", "contrast", "pairs"), nomatch = 2)[1]
@@ -376,20 +420,34 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
     }
     fam.info = c(misc$famSize, by.size, et)
     cnm = NULL
+    adjust = tolower(adjust)
     
     # get vcov matrix only if needed (adjust == "mvt")
-    corrmat = NULL
+    corrmat = sch.rank = NULL
     if (!is.na(pmatch(adjust, "mvt"))) {
         corrmat = cov2cor(vcov(object))
         attr(corrmat, "by.rows") = by.rows
     }
-    
+    else if (!is.na(pmatch(adjust, "scheffe"))) {
+        sch.rank = sapply(by.rows, function(.) qr(object@linfct[., , drop = FALSE])$rank)
+        if(length(unique(sch.rank)) > 1)
+            fam.info[1] = "uneven"   # This forces ragged.by = TRUE in .adj functions
+    }
+
     if(infer[1]) { # add CIs
-        acv = .adj.critval(level, result$df, adjust, fam.info, side, corrmat, by.rows)
+        acv = .adj.critval(level, result$df, adjust, fam.info, side, corrmat, by.rows, sch.rank)
         ###adjust = acv$adjust # in older versions, I forced same adj method for tests
         cv = acv$cv
         cv = switch(side + 2, cbind(-Inf, cv), cbind(-cv, cv), cbind(-cv, Inf))
         cnm = if (zFlag) c("asymp.LCL", "asymp.UCL") else c("lower.CL","upper.CL")
+        if(!is.null(misc$.predFlag)) {
+            cnm = c("lower.PL", "upper.PL")
+            sigma = misc$sigma
+            mesg = c(mesg, paste0(
+                "Prediction intervals and SEs are based on an error SD of ", 
+                .fmt.sigma(sigma)))
+            estName = names(result)[1] = "prediction"
+        }
         result[[cnm[1]]] = result[[1]] + cv[, 1]*result$SE
         result[[cnm[2]]] = result[[1]] + cv[, 2]*result$SE
         mesg = c(mesg, paste("Confidence level used:", level), acv$mesg)
@@ -420,7 +478,7 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
         else {
             t.ratio = result[[tnm]] = (result[[1]] - null + side * delta) / result$SE            
         }
-        apv = .adj.p.value(t.ratio, result$df, adjust, fam.info, tail, corrmat, by.rows)
+        apv = .adj.p.value(t.ratio, result$df, adjust, fam.info, tail, corrmat, by.rows, sch.rank)
         adjust = apv$adjust   # in case it was abbreviated
         result$p.value = apv$pval
         mesg = c(mesg, apv$mesg)
@@ -435,6 +493,9 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
     if (inv) {
         result[["SE"]] = with(link, abs(mu.eta(result[[1]]) * result[["SE"]]))
         result[[1]] = with(link, linkinv(result[[1]]))
+        if(bias.adjust)
+            mesg = c(mesg, paste("Bias adjustment applied based on sigma =", 
+                                 .fmt.sigma(sigma)))
     }
     
     if (length(misc$avgd.over) > 0) {
@@ -450,6 +511,10 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
         misc$pri.vars = names(object@levels)
     attr(summ, "pri.vars") = setdiff(union(misc$pri.vars, misc$by.vars), by)
     attr(summ, "by.vars") = by
+    attr(summ, "adjust") = adjust
+    attr(summ, "side") = side
+    attr(summ, "delta") = delta
+    attr(summ, "type") = type
     attr(summ, "mesg") = unique(mesg)
     class(summ) = c("summary_emm", "data.frame")
     summ
@@ -459,15 +524,33 @@ summary.emmGrid <- function(object, infer, level, adjust, by, type, df,
 
 #' @rdname summary.emmGrid
 #' @method predict emmGrid
+#' @param interval Type of interval desired (partial matching is allowed): 
+#' \code{"none"} for no intervals,
+#'   otherwise confidence or prediction intervals with given arguments, 
+#'   via \code{\link{confint.emmGrid}}.
+#'   
 #' @export
 #' @return \code{predict} returns a vector of predictions for each row of \code{object@grid}.
-predict.emmGrid <- function(object, type, ...) {
+predict.emmGrid <- function(object, type, 
+                            interval = c("none", "confidence", "prediction"),
+                            level = 0.95,
+                            bias.adjust = get_emm_option("back.bias.adj"), sigma, 
+                            ...) 
+{
     # update with any "summary" options
     opt = get_emm_option("summary")
     if(!is.null(opt)) {
         opt$object = object
         object = do.call("update.emmGrid", opt)
     }
+
+    interval = match.arg(interval)
+    if (interval %in% c( "confidence", "prediction")) {
+        if (interval == "prediction")
+            object@misc$.predFlag = TRUE
+        return(confint.emmGrid(object, type = type, level = level, 
+                               bias.adjust = bias.adjust, sigma = sigma, ...))
+    }
     
     if (missing(type))
         type = .get.predict.type(object@misc)
@@ -478,12 +561,17 @@ predict.emmGrid <- function(object, type, ...) {
     if ((type == "response") && (!is.null(object@misc$tran2)))
         object = regrid(object, transform = "mu")
     
-    pred = .est.se.df(object, do.se=FALSE)
+    pred = .est.se.df(object, do.se = FALSE, ...)
     result = pred[[1]]
     
     if (type %in% c("response", "mu", "unlink")) {
         link = attr(pred, "link")
         if (!is.null(link)) {
+            if (bias.adjust) {
+                if(missing(sigma))
+                    sigma = object@misc$sigma
+                link = .make.bias.adj.link(link, sigma)
+            }
             result = link$linkinv(result)
             if (is.logical(link$unknown) && link$unknown)
                 warning("Unknown transformation: \"", link$name, "\" -- no transformation applied.")
@@ -532,7 +620,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
 
 
 # Workhorse for getting estimates etc.
-.est.se.df = function(object, do.se=TRUE, tol = get_emm_option("estble.tol")) {
+.est.se.df = function(object, do.se=TRUE, tol = get_emm_option("estble.tol"), ...) {
     if (nrow(object@grid) == 0) {
         result = data.frame(NA, NA, NA)
         names(result) = c(object@misc$estName, "SE", "df")
@@ -543,7 +631,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
 
     if (!is.null(hook <- misc$estHook)) {
         if (is.character(hook)) hook = get(hook)
-        result = hook(object, do.se=do.se, tol=tol)
+        result = hook(object, do.se=do.se, tol=tol, ...)
     }
     else {
         active = which(!is.na(object@bhat))
@@ -570,8 +658,16 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     result[1] = as.numeric(result[1]) # silly bit of code to avoid getting a data.frame of logicals if all are NA
     result = as.data.frame(result)
     names(result) = c(misc$estName, "SE", "df")
-    if (!is.null(misc$tran) && is.character(misc$tran) && (misc$tran != "none")) {
+    if (!is.null(misc$.predFlag)) {
+        if (is.null(misc$sigma))
+            stop("No 'sigma' is available for obtaining Prediction intervals.", 
+                 call. = FALSE)
+        result$SE = sqrt(result$SE^2 + misc$sigma^2)
+    }
+    if (!is.null(misc$tran)) {
         attr(result, "link") = .get.link(misc)
+        if(is.character(misc$tran) && (misc$tran == "none"))
+            attr(result, "link") = NULL
     }
     result
 }
@@ -585,14 +681,20 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     else 
         NULL
     
-    if (is.list(link)) {  # See if multiple of link is requested
-        if (!is.null(misc$tran.mult))
-            link$mult = misc$tran.mult
-        if (!is.null(link$mult))
+    if (is.list(link)) {  # See if multiple of link is requested, or variable is offset
+        if (!is.null(misc$tran.mult) || !is.null(misc$tran.offset)) {
+            name = link$name
+            mult = link$mult = ifelse(is.null(misc$tran.mult), 1, misc$tran.mult)
+            off = link$offset = ifelse(is.null(misc$tran.offset), 0, misc$tran.offset)
             link = with(link, list(
-                linkinv = function(eta) linkinv(eta / mult),
-                mu.eta = function(eta) mu.eta(eta / mult) / mult,
-                name = paste0(round(mult, 3), "*", name)))
+                linkinv = function(eta) linkinv(eta / mult) - offset,
+                mu.eta = function(eta) mu.eta(eta / mult) / mult))
+            if(mult != 1)
+                name =  paste0(round(mult, 3), "*", name)        
+            if(off != 0)
+                name = paste0(name, "(mu + ", round(off, 3), ")")
+            link$name = name         
+        }
     }
     
     if (!is.null(link) && is.null(link$name))
@@ -600,12 +702,31 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     link
 }
 
+# patch-in alternative back-transform stuff for bias adjustment
+# Currently, we just use a 2nd-order approx for everybody:
+#   E(h(nu + E))  ~=  h(nu) + 0.5*h"(nu)*var(E)
+.make.bias.adj.link = function(link, sigma) {
+    if (is.null(sigma))
+        stop("Must specify 'sigma' to obtain bias-adjusted back transformations", call. = FALSE)
+    link$inv = link$linkinv
+    link$der = link$mu.eta
+    link$sigma22 = sigma^2 / 2
+    link$der2 = function(eta) with(link, 1000 * (der(eta + .0005) - der(eta - .0005)))
+    link$linkinv = function(eta) with(link, inv(eta) + sigma22 * der2(eta))
+    link$mu.eta = function(eta) with(link, der(eta) +
+                                         1000 * sigma22 * (der2(eta + .0005) - der2(eta - .0005)))
+    link
+}
+####!!!!! TODO: Re-think whether we are handling Scheffe adjustments correctly
+####!!!!!       if/when we shift around 'by' specs, etc.
+
 # utility to compute an adjusted p value
 # tail is -1, 0, 1 for left, two-sided, or right
 # Note fam.info is c(famsize, ncontr, estTypeIndex)
-# 2.14: added corrmat arg, dunnettx & mvt adjustments
+# lsmeans >= 2.14: added corrmat arg, dunnettx & mvt adjustments
+# emmeans > 1.3.4: we have sch.rank of same length as by.rows
 # NOTE: corrmat is NULL unless adjust == "mvt"
-.adj.p.value = function(t, DF, adjust, fam.info, tail, corrmat, by.rows) {
+.adj.p.value = function(t, DF, adjust, fam.info, tail, corrmat, by.rows, sch.rank) {
     fam.size = fam.info[1]
     n.contr = fam.info[2]
     et = as.numeric(fam.info[3])
@@ -624,7 +745,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     if ((et != 3) && adjust == "tukey") # not pairwise
         adjust = "sidak"
     
-    ragged.by = (is.character(fam.size) || adjust == "mvt")   # flag that we need to do groups separately
+    ragged.by = (is.character(fam.size) || adjust %in% c("mvt", p.adjust.methods))   # flag that we need to do groups separately
     if (!ragged.by)
         by.rows = list(seq_along(t))       # not ragged, we can do all as one by group
     
@@ -633,13 +754,16 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     
     # if estType is "prediction", use #contrasts + 1 as family size
     # (produces right Scheffe CV; Tukey ones are a bit strange)
-    scheffe.adj = ifelse(et == 1, 0, - 1)
+    # deprecated - used to try to keep track of scheffe.adj = ifelse(et == 1, 0, - 1)
     if (tail == 0)
         p.unadj = 2*pt(abs(t), DF, lower.tail=FALSE)
     else
         p.unadj = pt(t, DF, lower.tail = (tail<0))
     
-    pvals = lapply(by.rows, function(rows) {
+#    pvals = lapply(by.rows, function(rows) {
+    pval = numeric(length(t))
+    for(jj in seq_along(by.rows)) { ####(rows in by.rows) {
+        rows = by.rows[[jj]]
         unadj.p = p.unadj[rows]
         abst = abs(t[rows])
         df = DF[rows]
@@ -649,23 +773,22 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
         }
         if (adjust %in% p.adjust.methods) {
             if (n.contr == length(unadj.p))
-                pval = p.adjust(unadj.p, adjust, n = n.contr)
+                pval[rows] = p.adjust(unadj.p, adjust, n = n.contr)
             else # only will happen when by.rows is length 1
-                pval = as.numeric(apply(matrix(unadj.p, nrow=n.contr), 2, 
+                pval[rows] = as.numeric(apply(matrix(unadj.p, nrow=n.contr), 2, 
                                         function(pp) p.adjust(pp, adjust, n=sum(!is.na(pp)))))
         }
-        else pval = switch(adjust,
+        else pval[rows] = switch(adjust,
                            sidak = 1 - (1 - unadj.p)^n.contr,
                            # NOTE: tukey, scheffe, dunnettx all assumed 2-sided!
                            tukey = ptukey(sqrt(2)*abst, fam.size, zapsmall(df), lower.tail=FALSE),
-                           scheffe = pf(t[rows]^2 / (fam.size + scheffe.adj), fam.size + scheffe.adj, 
+                           scheffe = pf(t[rows]^2 / (sch.rank[jj]), sch.rank[jj], 
                                         df, lower.tail = FALSE),
                            dunnettx = 1 - .pdunnx(abst, n.contr, df),
                            mvt = 1 - .my.pmvt(t[rows], df, corrmat[rows,rows,drop=FALSE], -tail) # tricky - reverse the tail because we're subtracting from 1 
         )
-    })
-    pval = unlist(pvals)
-    
+    }
+
     chk.adj = match(adjust, c("none", "tukey", "scheffe"), nomatch = 99)
     
     if (ragged.by) {
@@ -676,7 +799,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     else {
         nc = n.contr
         fs = fam.size
-        scheffe.dim = fs + scheffe.adj
+        scheffe.dim = sch.rank[1]
     }
     do.msg = (chk.adj > 1) && (nc > 1) && !((fs < 3) && (chk.adj < 10)) 
     if (do.msg) {
@@ -690,14 +813,15 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
         mesg = paste("P value adjustment:", adjust, "method", xtra)
     }
     else mesg = NULL
-    list(pval=pval, mesg=mesg, adjust=adjust)
+    list(pval = pval, mesg = mesg, adjust = adjust)
 }
 
 # Code needed for an adjusted critical value
 # returns a list similar to .adj.p.value
-# 2.14: Added tail & corrmat args, dunnettx & mvt adjustments
+# lsmeans >= 2.14: Added tail & corrmat args, dunnettx & mvt adjustments
+# emmeans > 1.3.4: Added sch.rank
 # NOTE: corrmat is NULL unless adjust == "mvt"
-.adj.critval = function(level, DF, adjust, fam.info, tail, corrmat, by.rows) {
+.adj.critval = function(level, DF, adjust, fam.info, tail, corrmat, by.rows, sch.rank) {
     mesg = NULL
     
     fam.size = fam.info[1]
@@ -714,7 +838,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
         k = which(adj.meths == "bonferroni") 
     adjust = adj.meths[k]
     
-    if (!ragged.by && adjust != "mvt")
+    if (!ragged.by && (adjust != "mvt") && (length(unique(DF)) == 1))
         by.rows = list(seq_along(DF))       # not ragged, we can do all as one by group
     
     if ((et != 3) && adjust == "tukey") # not pairwise
@@ -726,7 +850,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     
     # asymptotic results when df is NA
     DF[is.na(DF)] = Inf
-    scheffe.adj = ifelse(et == 1, 0, - 1)
+    #### No longer used scheffe.adj = ifelse(et == 1, 0, - 1)
     
     chk.adj = match(adjust, c("none", "tukey", "scheffe"), nomatch = 99)
     if (ragged.by) {
@@ -737,7 +861,7 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     else {
         nc = n.contr
         fs = fam.size
-        scheffe.dim = fs + scheffe.adj
+        scheffe.dim = sch.rank[1]
     }
     do.msg = (chk.adj > 1) && (nc > 1) && 
         !((fs < 3) && (chk.adj < 10)) 
@@ -755,24 +879,27 @@ as.data.frame.emmGrid = function(x, row.names = NULL, optional = FALSE, ...) {
     
     adiv = ifelse(tail == 0, 2, 1) # divisor for alpha where needed
     
-    cvs = lapply(by.rows, function(rows) {
+    ###cvs = lapply(by.rows, function(rows) {
+    cv = numeric(sum(sapply(by.rows, length)))
+    for (jj in seq_along(by.rows)) { ####(rows in by.rows) {
+        rows = by.rows[[jj]]
         df = DF[rows]
         if (ragged.by) {
             n.contr = length(rows)
             fam.size = (1 + sqrt(1 + 8*n.contr)) / 2   # tukey family size - e.g., 6 pairs -> family of 4
         }
-        switch(adjust,
+        cv[rows] = switch(adjust,
                none = -qt((1-level)/adiv, df),
                sidak = -qt((1 - level^(1/n.contr))/adiv, df),
                bonferroni = -qt((1-level)/n.contr/adiv, df),
                tukey = qtukey(level, fam.size, df) / sqrt(2),
-               scheffe = sqrt((fam.size + scheffe.adj) * qf(level, fam.size + scheffe.adj, df)),
+               scheffe = sqrt((sch.rank[jj]) * qf(level, sch.rank[jj], df)),
                dunnettx = .qdunnx(level, n.contr, df),
-               mvt = .my.qmvt(level, df, corrmat[rows,rows,drop=FALSE], tail)
+               mvt = .my.qmvt(level, df, corrmat[rows, rows, drop = FALSE], tail)
         )
-    })
+    }
     
-    list(cv = unlist(cvs), mesg = mesg, adjust = adjust)
+    list(cv = cv, mesg = mesg, adjust = adjust)
 }
 
 
@@ -932,7 +1059,7 @@ print.summary_emm = function(x, ..., digits=NULL, quote=FALSE, right=TRUE) {
     if (!is.null(x$z.ratio)) 
         x$z.ratio = format(round(x$z.ratio, 3), nsmall = 3, sci = FALSE)
     if (!is.null(x$p.value)) {
-        fp = x$p.value = format(round(x$p.value,4), nsmall=4, sci=FALSE)
+        fp = x$p.value = format(round(x$p.value, 4), nsmall = 4, sci = FALSE)
         x$p.value[fp=="0.0000"] = "<.0001"
     }
     estn = attr(x, "estName")
@@ -966,7 +1093,7 @@ print.summary_emm = function(x, ..., digits=NULL, quote=FALSE, right=TRUE) {
     }
     else { # separate listing for each by variable
         m = .just.labs(m[, setdiff(names(x), by.vars), drop = FALSE], just)
-        pargs = as.list(x[,by.vars, drop=FALSE])
+        pargs = unname(as.list(x[,by.vars, drop=FALSE]))
         pargs$sep = ", "
         lbls = do.call(paste, pargs)
         for (lb in unique(lbls)) {
@@ -989,7 +1116,8 @@ print.summary_emm = function(x, ..., digits=NULL, quote=FALSE, right=TRUE) {
 # (but always at least 3)
 .opt.dig = function(x) {
     z = range(x) / max(abs(x))
-    zz = round(1.51 - log(diff(z), 10))  # approx 1 - log(diff(z/3))
+    dz = zapsmall(c(diff(z), z), digits = 8)[1]
+    zz = round(1.51 - log(dz, 10))  # approx 1 - log(diff(z/3))
     zz[is.infinite(zz)] = 3   # we get z = Inf when SE is 0
     max(zz, 3, na.rm = TRUE)  
 }
